"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlainWebviewComponent = exports.EditorWebviewComponent = exports.WebviewServiceImpl = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const error_1 = require("@opensumi/ide-core-common/lib/error");
const ide_editor_1 = require("@opensumi/ide-editor");
const browser_1 = require("@opensumi/ide-editor/lib/browser");
const color_registry_1 = require("@opensumi/ide-theme/lib/common/color-registry");
const editor_webview_1 = require("./editor-webview");
const electron_webview_webview_1 = require("./electron-webview-webview");
const iframe_webview_1 = require("./iframe-webview");
const plain_webview_1 = require("./plain-webview");
const types_1 = require("./types");
const webview_window_1 = require("./webview-window");
const { addElement } = ide_core_browser_1.arrays;
let WebviewServiceImpl = class WebviewServiceImpl {
    constructor() {
        this.webviewIdCount = 0;
        this.editorWebviewIdCount = 0;
        this.editorWebviewComponents = new Map();
        this.plainWebviewsComponents = new Map();
        this.webviews = new Map();
        this._revivers = [];
        this.storage = this.getStorage(new ide_core_browser_1.URI('editor-webview').withScheme(ide_core_browser_1.STORAGE_SCHEMA.SCOPE));
    }
    async tryReviveWebviewComponent(id) {
        if (this._revivers.length > 0) {
            let targetReviver;
            for (const reviver of this._revivers) {
                try {
                    const weight = await reviver.handles(id);
                    if (weight >= 0 && (!targetReviver || targetReviver.weight < weight)) {
                        targetReviver = {
                            weight,
                            reviver,
                        };
                    }
                }
                catch (e) {
                    this.logger.error(e);
                }
            }
            if (targetReviver) {
                try {
                    await targetReviver.reviver.revive(id);
                    return;
                }
                catch (e) {
                    this.logger.error(e);
                }
            }
        }
        throw new Error('Cannot revive webview ' + id);
    }
    registerWebviewReviver(reviver) {
        return addElement(this._revivers, reviver);
    }
    createPlainWebview(options = {}) {
        if (this.appConfig.isElectronRenderer) {
            if (options.preferredImpl && options.preferredImpl === 'iframe') {
                return new plain_webview_1.IframePlainWebview();
            }
            return new plain_webview_1.ElectronPlainWebview();
        }
        else {
            if (options.preferredImpl && options.preferredImpl === 'webview') {
                (0, ide_core_browser_1.getDebugLogger)().warn((0, ide_core_browser_1.localize)('webview.webviewTagUnavailable', 'Webview is unsupported on non-electron env, please use iframe instead.'));
            }
            return new plain_webview_1.IframePlainWebview();
        }
    }
    createWebview(options) {
        let webview;
        if (this.appConfig.isElectronRenderer) {
            webview = this.injector.get(electron_webview_webview_1.ElectronWebviewWebviewPanel, [(this.webviewIdCount++).toString(), options]);
        }
        else {
            webview = this.injector.get(iframe_webview_1.IFrameWebviewPanel, [(this.webviewIdCount++).toString(), options]);
        }
        this.webviews.set(webview.id, webview);
        webview.onRemove(() => {
            this.webviews.delete(webview.id);
        });
        return webview;
    }
    getWebview(id) {
        return this.webviews.get(id);
    }
    async storeWebviewResource(id) {
        return this.storage.then((storage) => {
            if (this.editorWebviewComponents.has(id)) {
                const editorWebview = this.editorWebviewComponents.get(id);
                const res = { v: 2, resource: editorWebview.resource, metadata: editorWebview.metadata };
                storage.set(id, JSON.stringify(res));
            }
            else {
                storage.delete(id);
            }
        });
    }
    async tryRestoredWebviewComponent(id) {
        var _a, _b;
        const storage = await this.storage;
        const res = storage.get(id) ? JSON.parse(storage.get(id)) : null;
        if (!res) {
            return;
        }
        let resource;
        let metadata;
        if ((res === null || res === void 0 ? void 0 : res.v) === 2) {
            resource = res.resource;
            metadata = res.metadata;
        }
        else {
            resource = res;
        }
        if (resource) {
            const component = this.createEditorWebviewComponent((_a = resource.metadata) === null || _a === void 0 ? void 0 : _a.options, (_b = resource.metadata) === null || _b === void 0 ? void 0 : _b.id, metadata);
            component.title = resource.name;
            component.supportsRevive = !!resource.supportsRevive;
            this.tryReviveWebviewComponent(id);
        }
    }
    createEditorWebviewComponent(options, id, metadata) {
        if (!id) {
            id = (this.editorWebviewIdCount++).toString();
        }
        if (this.editorWebviewComponents.has(id)) {
            return this.editorWebviewComponents.get(id);
        }
        const component = this.injector.get(EditorWebviewComponent, [
            id,
            () => this.createWebview(options),
            metadata,
        ]);
        this.editorWebviewComponents.set(id, component);
        component.addDispose({
            dispose: () => {
                this.editorWebviewComponents.delete(id);
            },
        });
        component.onDidUpdateResource(() => {
            this.storeWebviewResource(id);
        });
        return component;
    }
    createEditorPlainWebviewComponent(options = {}, id) {
        id = id || (this.editorWebviewIdCount++).toString();
        if (this.editorWebviewComponents.has(id)) {
            return this.editorWebviewComponents.get(id);
        }
        const component = this.injector.get(EditorWebviewComponent, [
            id,
            () => this.createPlainWebview(options),
        ]);
        this.editorWebviewComponents.set(id, component);
        component.addDispose({
            dispose: () => {
                this.editorWebviewComponents.delete(id);
            },
        });
        return component;
    }
    getWebviewThemeData(theme) {
        const editorFontFamily = this.editorPreferences['editor.fontFamily'];
        const editorFontWeight = this.editorPreferences['editor.fontFamily'];
        const editorFontSize = this.editorPreferences['editor.fontSize'];
        const exportedColors = (0, color_registry_1.getColorRegistry)()
            .getColors()
            .reduce((colors, entry) => {
            const color = theme.getColor(entry.id);
            if (color) {
                colors['vscode-' + entry.id.replace('.', '-')] = color.toString();
            }
            return colors;
        }, {});
        const styles = Object.assign({ 'vscode-font-family': '-apple-system, BlinkMacSystemFont, "Segoe WPC", "Segoe UI", system-ui, "Ubuntu", "Droid Sans", sans-serif', 'vscode-font-weight': 'normal', 'vscode-font-size': '13px', 'vscode-editor-font-family': editorFontFamily, 'vscode-editor-font-weight': editorFontWeight, 'vscode-editor-font-size': editorFontSize }, exportedColors);
        const activeTheme = ApiThemeClassName.fromTheme(theme);
        return { styles, activeTheme };
    }
    getOrCreatePlainWebviewComponent(id, options) {
        if (!this.plainWebviewsComponents.has(id)) {
            const webview = this.createPlainWebview(options);
            const component = this.injector.get(PlainWebviewComponent, [id, webview]);
            this.plainWebviewsComponents.set(id, component);
            component.onDispose(() => {
                this.plainWebviewsComponents.delete(id);
            });
        }
        return this.plainWebviewsComponents.get(id);
    }
    getEditorPlainWebviewComponent(id) {
        const component = this.editorWebviewComponents.get(id);
        if (component && component.webview.loadURL) {
            return component;
        }
    }
    getPlainWebviewComponent(id) {
        return this.plainWebviewsComponents.get(id);
    }
    createWebviewWindow(options, env) {
        if (this.appConfig.isElectronRenderer) {
            return this.injector.get(webview_window_1.ElectronPlainWebviewWindow, [options, env]);
        }
        (0, error_1.throwNonElectronError)('WebviewServiceImpl.createWebviewWindow');
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], WebviewServiceImpl.prototype, "injector", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.AppConfig),
    tslib_1.__metadata("design:type", Object)
], WebviewServiceImpl.prototype, "appConfig", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(browser_1.EditorPreferences),
    tslib_1.__metadata("design:type", Object)
], WebviewServiceImpl.prototype, "editorPreferences", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.ILogger),
    tslib_1.__metadata("design:type", Object)
], WebviewServiceImpl.prototype, "logger", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.StorageProvider),
    tslib_1.__metadata("design:type", Function)
], WebviewServiceImpl.prototype, "getStorage", void 0);
WebviewServiceImpl = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], WebviewServiceImpl);
exports.WebviewServiceImpl = WebviewServiceImpl;
var ApiThemeClassName;
(function (ApiThemeClassName) {
    ApiThemeClassName["light"] = "vscode-light";
    ApiThemeClassName["dark"] = "vscode-dark";
    ApiThemeClassName["highContrast"] = "vscode-high-contrast";
    ApiThemeClassName["highContrastLight"] = "vscode-high-contrast-light";
})(ApiThemeClassName || (ApiThemeClassName = {}));
(function (ApiThemeClassName) {
    function fromTheme(theme) {
        switch (theme.type) {
            case 'light':
                return ApiThemeClassName.light;
            case 'dark':
                return ApiThemeClassName.dark;
            case 'hcDark':
                return ApiThemeClassName.highContrast;
            case 'hcLight':
                return ApiThemeClassName.highContrastLight;
        }
    }
    ApiThemeClassName.fromTheme = fromTheme;
})(ApiThemeClassName || (ApiThemeClassName = {}));
let EditorWebviewComponent = class EditorWebviewComponent extends ide_core_browser_1.Disposable {
    get supportsRevive() {
        return this._supportsRevive;
    }
    set supportsRevive(value) {
        this._supportsRevive = value;
        this.eventBus.fire(new ide_editor_1.ResourceNeedUpdateEvent(this.webviewUri));
        this._onDidUpdateResource.fire(this.resource);
    }
    open(options) {
        return this.workbenchEditorService.open(this.webviewUri, Object.assign(Object.assign({}, options), { preview: false }));
    }
    close() {
        this.workbenchEditorService.closeAll(this.webviewUri);
    }
    get icon() {
        return this._icon;
    }
    set icon(icon) {
        this._icon = icon;
        this.eventBus.fire(new ide_editor_1.ResourceNeedUpdateEvent(this.webviewUri));
        this._onDidUpdateResource.fire(this.resource);
    }
    get title() {
        return this._title;
    }
    set title(title) {
        this._title = title;
        this.eventBus.fire(new ide_editor_1.ResourceNeedUpdateEvent(this.webviewUri));
        this._onDidUpdateResource.fire(this.resource);
    }
    get webview() {
        if (!this._webview) {
            this.createWebview();
        }
        return this._webview;
    }
    get resource() {
        return {
            icon: this.icon,
            name: this.title,
            uri: this.webviewUri,
            metadata: {
                id: this.id,
                options: this.webview.options,
            },
            supportsRevive: this.supportsRevive,
        };
    }
    get webviewUri() {
        return ide_core_browser_1.URI.from({
            scheme: types_1.EDITOR_WEBVIEW_SCHEME,
            path: this.id,
        });
    }
    get editorGroup() {
        const uri = this.webviewUri;
        return this.workbenchEditorService.editorGroups.find((g) => g.resources.findIndex((r) => r.uri.isEqual(uri)) !== -1);
    }
    get group() {
        return this.editorGroup;
    }
    get componentId() {
        return types_1.EDITOR_WEBVIEW_SCHEME + '_' + this.id;
    }
    constructor(id, webviewFactory, metadata) {
        super();
        this.id = id;
        this.webviewFactory = webviewFactory;
        this.metadata = metadata;
        this._onDidUpdateResource = new ide_core_browser_1.Emitter();
        this.onDidUpdateResource = this._onDidUpdateResource.event;
        this._onDidChangeGroupIndex = new ide_core_browser_1.Emitter();
        this.onDidChangeGroupIndex = this._onDidChangeGroupIndex.event;
        this._supportsRevive = false;
        this._title = 'Webview';
        this._icon = '';
        const componentId = types_1.EDITOR_WEBVIEW_SCHEME + '_' + this.id;
        this.addDispose(this.editorComponentRegistry.registerEditorComponent({
            scheme: types_1.EDITOR_WEBVIEW_SCHEME,
            uid: componentId,
            component: editor_webview_1.EditorWebviewComponentView,
            renderMode: browser_1.EditorComponentRenderMode.ONE_PER_WORKBENCH,
            metadata,
        }));
        this.addDispose(this.editorComponentRegistry.registerEditorComponentResolver(types_1.EDITOR_WEBVIEW_SCHEME, (resource, results) => {
            if (resource.uri.path.toString() === this.id) {
                results.push({
                    type: browser_1.EditorOpenType.component,
                    componentId,
                });
            }
        }));
        this.addDispose({
            dispose: () => {
                this.workbenchEditorService.closeAll(this.webviewUri, true);
            },
        });
        this.addDispose(this.eventBus.on(browser_1.EditorGroupChangeEvent, (e) => {
            var _a;
            if ((_a = e.payload.newResource) === null || _a === void 0 ? void 0 : _a.uri.isEqual(this.webviewUri)) {
                this._onDidChangeGroupIndex.fire(e.payload.group.index);
            }
        }));
    }
    createWebview() {
        this._webview = this.webviewFactory();
        this.addDispose(this._webview);
        if (typeof this._webview.onDidFocus === 'function') {
            this.addDispose(this._webview.onDidFocus(() => {
                if (this.editorGroup) {
                    this.editorGroup.gainFocus();
                }
            }));
        }
        return this._webview;
    }
    clear() {
        const componentId = types_1.EDITOR_WEBVIEW_SCHEME + '_' + this.id;
        this.editorComponentRegistry.clearPerWorkbenchComponentCache(componentId);
        this.webview.remove();
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(),
    tslib_1.__metadata("design:type", ide_editor_1.WorkbenchEditorService)
], EditorWebviewComponent.prototype, "workbenchEditorService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(),
    tslib_1.__metadata("design:type", browser_1.EditorComponentRegistry)
], EditorWebviewComponent.prototype, "editorComponentRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.IEventBus),
    tslib_1.__metadata("design:type", Object)
], EditorWebviewComponent.prototype, "eventBus", void 0);
EditorWebviewComponent = tslib_1.__decorate([
    (0, di_1.Injectable)({ multiple: true }),
    tslib_1.__metadata("design:paramtypes", [String, Function, Object])
], EditorWebviewComponent);
exports.EditorWebviewComponent = EditorWebviewComponent;
let PlainWebviewComponent = class PlainWebviewComponent extends ide_core_browser_1.Disposable {
    constructor(id, webview) {
        super();
        this.id = id;
        this.webview = webview;
        this.addDispose(this.webview);
    }
};
PlainWebviewComponent = tslib_1.__decorate([
    (0, di_1.Injectable)({ multiple: true }),
    tslib_1.__metadata("design:paramtypes", [String, Object])
], PlainWebviewComponent);
exports.PlainWebviewComponent = PlainWebviewComponent;
//# sourceMappingURL=webview.service.js.map